#ifndef GL_SGG_H
#define GL_SGG_H


#include <sgg.h>
#include <nehe.h>
#include <win_input.h>
#include <sven.h>

namespace sgg {

using namespace sven;
using namespace sgg_tools;
using nehe::TextFormat;
using nehe::neheTexture;
using namespace std_ex;

struct Drawable_Rect : public Float_Rect, public Drawable {
	Drawable_Rect(float tx, float ty, float bx, float by) : Float_Rect(tx,ty,bx,by) {}
	Drawable_Rect(Float_Rect f) : Float_Rect(f) {}
	void draw();
};

struct Drawable_Rects : public vector<Drawable_Rect>, public Drawable {
	void draw() {  
		iterator t=begin(); 
		while(t!=end()) { 
			t->draw(); 
			t++;
		}
	}
};

inline void goto4by3() {
	glMatrixMode (GL_MODELVIEW);
	glLoadIdentity();
	glTranslatef(0.0f,0.0f,-7.2f);
}


/////////////////////////////////////////////////////////////////////////////
//Handy Drawable classes
//Drawable_Image, BackgroundImage, Drawable_String, Drawable_Rect


////////////////////////////////////////////////
//Drawable Image
//The constructor specifies the top and bottom corners of a rectangle,
//as well as the name of an image to load as a texture.  The optional
//z ofset can be used to establish an effective screen depth.
//suggested z values are 0.01 * n.

class GLimage {
	
	neheTexture texture;
	float tx, ty, bx, by, z;

public:

	GLimage(float tx, float ty, float bx, float by, const char * fname, float z=0) :
	tx(tx), ty(ty), bx(bx), by(by), z(z), texture(fname) {}

	GLimage(float w, float h, const char * fname, float z=0) 
		: tx(w*-.5f), ty(h*.5f), bx(w*.5f), by(h*-.5f), z(z), texture(fname) {}

	GLimage(float w, float h, neheTexture tex, float z=0) 
		: tx(w*-.5f), ty(h*.5f), bx(w*.5f), by(h*-.5f), z(z), texture(tex) {}


	void scale(float s) {
		float ys = (ty-by*.5)*s;
		ty+=ys;
		by-=ys;
		float xs = (bx-tx*.5)*s;
		tx-=xs;
		bx+=xs;
	}


	//So I could have used a display list.
	//I haven't, partly because I heard a rumor that display lists actually slow
	//down the nvidia cards (though apparently they work great with SGI hardware).
	//I suppose I should really get around to testing this myself...
	void draw() {
		if(z) {
			glPushMatrix();
			glTranslatef(0.0f,0.0f,z);
		}


		glBindTexture(GL_TEXTURE_2D,texture);	

		/*glBegin(GL_QUADS);
			glTexCoord2f(0.0f,0.0f); 
			glVertex3f(tx,by,0.0f); 
			
			glTexCoord2f(1.0f, 0.0f); 
			glVertex3f(bx,by,0.0f); 
			
			glTexCoord2f(1.0f,1.0f); 
			glVertex3f(bx,ty,0.0f); 
			
			glTexCoord2f(0.0f,1.0f); 
			glVertex3f(tx,ty,0.0f); 
		glEnd();
*/

		glBegin(GL_TRIANGLE_STRIP);		
			glTexCoord2d(1,1); glVertex3f(bx,ty,0); // Top Right
			glTexCoord2d(0,1); glVertex3f(tx,ty,0); // Top Left
			glTexCoord2d(1,0); glVertex3f(bx,by,0); // Bottom Right
			glTexCoord2d(0,0); glVertex3f(tx,by,0); // Bottom Left
		glEnd();

		if(z) {
			glPopMatrix();
		}
	}
};

struct Drawable_Image : public Drawable, public GLimage {
	Drawable_Image(float tx, float ty, float bx, float by, const char * fname, float z=0) 
		: GLimage(tx,ty,bx,by,fname,z) {}

	Drawable_Image(GLimage i) : GLimage(i) {}

	void draw() { GLimage::draw(); }
};

////////////////////////////////////////////////////////////////////
//BackGroundImage
//so it is generally assumed that anyone using a Screen3v will
//probably want to load one (and only one) background image.
//The obvious alternative, is to overload your refresh function to call
//a clear, before drawing things.
struct BackGroundImage : public Drawable_Image {
	BackGroundImage(const char * fname) : Drawable_Image(-4.0f,3.0f,4.0f,-3.0f,fname, -.1f) {}
};

//If you find yourself needing to use an array of Pick_Behavior *'s,
//you may need to dig up the old non-templated code (included below)
template <class PB> //PB stands for PickingBehavior
struct Pickable_Area_T : protected Float2_PolygonList, public Pickable,
	public PB {

	//Pickable_Area_T(Float2_PolygonList sp, PB pb=PB()) : 
	 // Float2_PolygonList(sp), PB(pb) {}

	//What I want is this (see 3e::373): 

    //index_error is thrown if we can't find an area with a matching name
	//Pickable_Area_T(const PickingAreas &pa, const string &name) 
	//	try : Float2_PolygonList(pa.polys[ex_index(name,pa.names)]) {}
	//    catch(index_failed_error) { throw pa_area_undefined(); } 


	//But MSVC doesn't like that, so I need to settle with this, and
	//make a note to the effect that this constructor returns a 
	//index2_failed_error<string> if the name isn't found.
	Pickable_Area_T(const PickingAreas &pa, const string &name, PB pb=PB()) : 
		Float2_PolygonList(pa.polys[ex_index2(name,pa.names)]), PB(pb) {}

	void p_draw() {
		for(Float2_PolygonList::const_iterator p=begin();p!=end();p++) {
			glBegin(GL_POLYGON);
				for(int i=0;i<p->size();i++) glVertex3f((*p)[i].x,(*p)[i].y,0);
			glEnd();
		}
	}

	void pick() { PB::pick(); }

};

struct Pickable_Area : public Pickable, protected Float2_PolygonList {
private:
  Pick_Behavior *pick_fun;
public:

	Pickable_Area(Float2_PolygonList sp, Pick_Behavior * pick_fun) : 
	  Float2_PolygonList(sp), pick_fun(pick_fun) {}

  //so at the moment, I am assuming that when the Pickable_Area's
  //p_draw function is called, our 4by3 matrix is ontop of the stack.
  //a safe assumption so long as everyone rembers to always leave
  //that matrix ontop of the stack...
  void p_draw() {
	for(Float2_PolygonList::const_iterator p=begin();p!=end();p++) {
		glBegin(GL_POLYGON);
			for(int i=0;i<p->size();i++) glVertex3f((*p)[i].x,(*p)[i].y,0);
		glEnd();
	}
  }

  void pick() { (*pick_fun)(); }

  virtual ~Pickable_Area() { delete pick_fun; }
};

struct Multi_Picker {
  virtual void p_draw(int base)=0;
  virtual void pick(int i)=0;
  virtual int size()=0;
  virtual ~Multi_Picker() {}
};


struct Pickable_Vector : public Multi_Picker, 
  public vector<Pickable *> {
  
  Pickable_Vector() {}

  Pickable_Vector (const vector<Pickable *> & v) :
    vector<Pickable *>(v) {}

  int size() { return vector<Pickable *>::size(); } 
  void p_draw(int base=0) {
    for(int i=0;i<size();i++) {
      glLoadName(i+base);
      (*this)[i]->p_draw();
    }
  }
  
  void pick(int p) {
    if(p!=-1) (*this)[p]->pick();
  }
};


//A Picker, as used by glPick(), is any class with a p_draw()
//and pick(int p) function.  Notice that if you pass a Pickable_Vector to
//glPick(), you will properly pick one of that objects Pickables.  However,
//if you pass in a MultiPicker containing a whole bunch of PickableVectors,
//then you will pick one of the PickableVectors' Pickable objects.
//Alternatively, you could pass in a couple of PickableVectors, and a few
//other types of specialized Multi_Picker_Elements.
//So, an example MultiPicker might consist of the following things:
//Pickable_Vector_Element * command_buttons,
//List_Element * reasearch_control,
//Pickable_Vector Element * stars.

//The standard (but not only) way to interfase with
//the glPick function is through a MultiPicker object.
//Notice the wierd possibilities of having a MultiPicker containing
//other MultiPickers.  I can't think of a reason why one would do this,
//but allowing for it is trivial, and I would rather err on the side of
//generality.
struct Multi_Multi_Picker : 
  public vector<Multi_Picker *>, 
  public Multi_Picker {
  
  int size() { return vector<Multi_Picker *>::size(); }
  void p_draw(int base=0) {
    int s=base;
    
	for(iterator i=begin();i!=end();i++) {
		(*i)->p_draw(s);
		s+=(*i)->size();
	}
  }


	void pick(int p) {
		for(iterator i=begin();i!=end();i++) {
			if(p >= (*i)->size()) { 
				p-=(*i)->size(); 
			}
			else {	(*i)->pick(p); break; }
		}
	}
};



//I have two obvious alternatives in making this pick function.  Either
//I can make an abstract class Picker, and pass in a Picker * p,
//or I can use templating.  The only problem with the templating approach
//is that I end up needing to inline the function to ensure proper 
//instantation....

//Calling this function with a given struct as Picker effecitively runs
//a compile time check to make sure that the struct has the member functions
//draw() and pick().  This way, I don't have to bother with the base class,
//or any of the pointer messiness required for run time polymorphism.  I also
//get the added benifit of being able to create Picker structs that use
//function objects or function pointers to implement draw as Pick.  See
//Fun_Picker.
template <class Picker> 
inline void glPick(Picker &p) {
	using namespace win_tools;
	using namespace nehe;

	GLuint	buffer[512];							// Set Up A Selection Buffer
	int hits=0;

	// The Size Of The Viewport. [0] Is <x>, [1] Is <y>, [2] Is <length>, [3] Is <width>
	GLint	viewport[4];

	// This Sets The Array <viewport> To The Size And Location Of The Screen Relative To The Window
	glGetIntegerv(GL_VIEWPORT, viewport);
	glSelectBuffer(512, buffer);						// Tell OpenGL To Use Our Array For Selection
	glRenderMode(GL_SELECT);	
	glInitNames();								// Initializes The Name Stack
	glPushName(0);
	glMatrixMode(GL_PROJECTION);						// Selects The Projection Matrix
	glPushMatrix();								// Push The Projection Matrix
	glLoadIdentity();							// Resets The Matrix	
	gluPickMatrix(g_mouse.x, viewport[3]-g_mouse.y, 15.0f, 15.0f, viewport);

	//this might need some editting....
	gluPerspective(PERSP_ANGLE,(GLfloat) (viewport[2]-viewport[0])/(GLfloat) (viewport[3]-viewport[1]),PERSP_NEAR,PERSP_FAR);

	glMatrixMode(GL_MODELVIEW);						// Select The Modelview Matrix
	glPushMatrix();
	p.p_draw();
	glPopMatrix();

	int choose = -1;

	glMatrixMode(GL_PROJECTION);						// Select The Projection Matrix
	glPopMatrix();								// Pop The Projection Matrix
	glMatrixMode(GL_MODELVIEW);						// Select The Modelview Matrix

	hits=glRenderMode(GL_RENDER);						// Switch To Render Mode, Find Out How Many

	if (hits > 0)								// If There Were More Than 0 Hits
	{
		choose = buffer[3];					// Make Our Selection The First Object
		int	depth = buffer[1];					// Store How Far Away It Is

		//fyi_msg("you got a hit");

		for (int loop = 1; loop < hits; loop++)				// Loop Through All The Detected Hits
		{
			// If This Object Is Closer To Us Than The One We Have Selected
			if (buffer[loop*4+1] < GLuint(depth))
			{
				choose = buffer[loop*4+3];			// Select The Closer Object
				depth = buffer[loop*4+1];			// Store How Far Away It Is
			}       
		}

	}

	if(choose!=-1) p.pick(choose);	
  
}

template <class container,class D>
inline container::iterator find_d(const D &d,container &c) {
	for(container::iterator i=c.begin();i!=c.end();i++) {
		if(**i==d) return i;
	}
	return c.end();
}

class Screen3v : public Screen {
protected:
	vector<Drawable *> drawables;
	Multi_Multi_Picker picker;
	vector<Refreshable *> refreshables;
	vector<Registerable *> kill_list;
	vector<Registerable *> kill_watch;
	Pickable_Vector * pickables;

public:
	struct reg_error {};

	Screen3v() { 
		goto4by3(); 
	    picker.push_back(pickables = new Pickable_Vector());
	}

	void reg(Registerable * r) {
		Drawable * d =		dynamic_cast<Drawable *>(r);
		Pickable * p =		dynamic_cast<Pickable *>(r);
		Refreshable * f =	dynamic_cast<Refreshable *>(r);
		Multi_Picker * mp = dynamic_cast<Multi_Picker *>(r);
		KillFlag * kf =		dynamic_cast<KillFlag *>(r);

		if(mp)		picker.push_back(dynamic_cast<Multi_Picker *>(r));
		else {
			if(p)	{
				//Pickable_Vector * pv=dynamic_cast<Pickable_Vector *>(*picker.begin());
				//if(pv) pv->push_back(reinterpret_cast<Pickable *>(r));
				//else throw reg_error();  //this should never happen, given the construtor.

				pickables->push_back(dynamic_cast<Pickable *>(r));

			}
		}
		if(d)		drawables.push_back(dynamic_cast<Drawable *>(r));
		if(f)		refreshables.push_back(dynamic_cast<Refreshable *>(r));
		if(kf)		kill_watch.push_back(r);
					kill_list.push_back(r);
		

	}


	virtual void refresh() {
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		
		foreach(drawables,Drawable::mem_draw());
		glPick(picker);
		foreach(refreshables,Refreshable::mem_refresh());
	
		//this chunck of code, with it's 5 dynamic casts, makes me very unhappy...
		for(int i=0;i<kill_watch.size();i++) {
			if(dynamic_cast<KillFlag *>(kill_watch[i])->killme) {

				vector<Registerable *>::iterator r = 
					find(kill_list.begin(), kill_list.end(), kill_watch[i]);

				vector<Drawable *>::iterator d = 
					find(drawables.begin(), drawables.end(), dynamic_cast<Drawable *>(kill_watch[i]));

				Multi_Multi_Picker::iterator mp = 
					find(picker.begin(), picker.end(),dynamic_cast<Multi_Picker *>(kill_watch[i]));

				Pickable_Vector::iterator p =
					find(pickables->begin(),pickables->end(), dynamic_cast<Pickable *>(kill_watch[i]));

				vector<Refreshable *>::iterator f =
					find(refreshables.begin(),refreshables.end(), dynamic_cast<Refreshable *>(kill_watch[i]));


				delete kill_watch[i];

				kill_watch.erase(kill_watch.begin()+i);
				kill_list.erase(r);

				if(d!=drawables.end()) drawables.erase(d);
				if(mp!=picker.end()) picker.erase(mp);
				if(p!=pickables->end()) pickables->erase(p);
				if(f!=refreshables.end()) refreshables.erase(f);
			}
			else { i++; }
		}

		/*for(killmap::iterator i= kill_watch.begin(); i!=kill_watch.end();i++) {
			if(i->first->killme) {
				delete i->first;
				kill_list.erase(i->second);
				kill_watch.erase(i);
			}
		}*/
	}

	static void del_reg(Registerable * r) { delete r; }
	virtual ~Screen3v() {
		for_each(kill_list.begin(),kill_list.end(),del_reg);
	}
};


struct SubScreen : public Pickable, public Refreshable, 
				   public Drawable_Rect, public Screen3v {

	SubScreen(float tx, float ty, float bx, float by) : Drawable_Rect(tx,ty,bx,by) {}
	SubScreen(Float_Rect f) : Drawable_Rect(f) {}

	//change the refresh function so that it doesn't do anything more than refresh everything
	virtual void refresh() {
		foreach(refreshables,Refreshable::mem_refresh());		
	}
	virtual void pick() {
		glPick(picker);	
	}
	virtual void draw() {
		foreach(drawables,Drawable::mem_draw());	
	}
	virtual void p_draw() {
		Drawable_Rect::draw();
	}

};

}

#include <gl_text_sgg.h>

#endif