//The messyness of setting up a windows app cannot be eliminated, but
//it can be hidden.
//If you are using SGG, all you should need to think about is setting up the screen stack.
//Thus the StdWinMain macro, which lets you avoid the messyness of a WinMain.
//Just pass in an init function (or a function object if you are into that stuff),
//and the rest will be taken care of for you.
//(though if you are doing things right, all your cleanup is implicit
//in the destructors for your classes.)


#ifndef GL_WIN_SGG_H
#define GL_WIN_SGG_H

#include <windows.h>
#include <sound.h>
#include <win_input.h>
#include <gl_sgg.h>
#include <pa.h>


namespace sgg {

	
struct QuitClick {
	void pick() { 
		using namespace win_tools;
		if(g_mouse.lClick()) g_quit_flag=true;
	}
};

struct CloseClick {
	void pick() { 
		using namespace win_tools;
		if(g_mouse.lClick()) g_close_screen=true;
	}
};

struct PlaySoundOnMouseOver { 
	string sound_file;
	PlaySoundOnMouseOver(string sound_file) : sound_file(sound_file) {}
	void pick() { using namespace win_tools; playSound(sound_file.c_str()); }
};

struct QuitCommand : public Refreshable {
	void refresh() {
		using namespace win_tools;
		if(keys[DIK_ESCAPE]) g_quit_flag=true;
	}
};


template <class Init>
inline int WINAPI WindowsMain(Init init, HINSTANCE hInst, HINSTANCE hPrevInstance, LPSTR lpszCmdLine, int nCmdShow)
		{

nehe::PERSP_NEAR = 2.0f;
nehe::PERSP_FAR = 100.0f;
nehe::PERSP_ANGLE =45.0f;

		using namespace win_tools;
		using namespace nehe;

		MSG msg;

		hInstance=hInst;  //Set the global hInstance variable, this will be used all over the place
		setWndProc(WI_Proc);


#ifdef CURSOR_LOADER
 		NeheGLWindow nehe_win("SGG App",1024,768,16,CURSOR_LOADER);	
#else 
		NeheGLWindow nehe_win("SGG App",1024,768,16);
#endif


		SoundResourceWrapper sound_wrapper;
#ifdef USE_DINPUT
		DI_ResourceWrapper di_wapper;
#endif
		
		init();

		while(!g_quit_flag) {	
			if (PeekMessage(&msg,NULL,0,0,PM_REMOVE)) { 
				if (msg.message == WM_QUIT) break;
				TranslateMessage(&msg);
				DispatchMessage(&msg);
			} 
			else {
#ifdef USE_DINPUT
				updateDirectInput();
#endif
				gui_update();

				SwapBuffers(hDC);
			}
		}

		return (msg.wParam);	
		}

}

#define StdWinMain(init) \
int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPrevInstance, LPSTR lpszCmdLine, int nCmdShow) {\
return sgg::WindowsMain(init, hInst,hPrevInstance,lpszCmdLine, nCmdShow); }


#endif