/////////////////////////////////////////////////////////////////////////
//So here are a bunch of quick class definitions that kinda frustrate me...
//In a perfect world, I would use some standard classes to do the work that
//I am doing with each of these little structs.  Sadly, niether STL nor BOOST
//seems to have anything that will do the trick right off the bat, and I
//can't use classes from other API's (like any Windows library or Nvidia's Cg) 
//without making myself dependant on more outside libraries than I would like...
//Thus I am defining a "Sven's Odds and Ends library" that holds
//definitions for all these sorts of little thingies.

#ifndef SVEN_H
#define SVEN_H

//This guy always wants to be at the very top of everything whenever 
//you are using STL strings with MSVC
#pragma warning(disable: 4786) 

#include <vector>
#include <iostream>
#include <sstream>
#include <map>
#include <string>
#include <fstream>
#include <algorithm>
#include <functional>


#include <floatStructs.h>

//handy defines
#ifndef PI
#define PI 3.1415926535897932384626433832795028841971693993751058209749445923078164062862089986280348253421170679
#endif

#ifndef RAD2DEG
#define RAD2DEG (180./PI)
#endif

#ifndef DEG2RAD
#define DEG2RAD (PI/180.)
#endif

typedef void (* void_fun )();



namespace std_ex {

using namespace std;

//standard error struct
//struct bad_input {};

//A less general but slightly shorter foreach function.
template <class container, class F>
inline void foreach(const container &v, F f) {
	for_each(v.begin(),v.end(),f);
}

//A less general but slightly shorter foreach function.
/*template <class container, class D>
inline container::const_iterator find(D &d, const container &v) {
	return std::find(v.begin(),v.end(),d);
}*/

template <class container, class D>
inline container::const_iterator find(D &d, const container &v) {
	return std::find(v.begin(),v.end(),d);
}

//A less general but slightly shorter foreach function.
//template < class T, class F>
//inline void foreach(const vector<T> &v, F f) {
//	for_each(v.begin(),v.end(),f);
//}

//index
//index is usefull if I have two matching vectors,
//one of whch I am using as a lookup for the other.
//I supose that if I were a good person, I would use
//some version of map in those situations, but I am
//not yet that fluent with STL.... TIMTOWTDI, right? :)
template <class D>
inline int index(const D &d, const vector<D> & v) {
	for(int i=0;i<v.size();i++) {
		if(d==v[i]) return i;
	}
	return -1;
}

//ex_index
//this version of index throws an exception if it can't find d.
struct index_failed_error {};
template <class D>
inline int ex_index(const D &d, const vector<D> & v) {
	for(int i=0;i<v.size();i++) {
		if(d==v[i]) return i;
	}
	throw index_failed_error();
	return -1;
}

//ex_index2
//this version throws a more informative exception.
//however, it is a little more complicated to use,
//so I only invoke it when I really need to 
//(like in the Picking_Area_T case).
template <class D>
struct index2_failed_error { D d; index2_failed_error(D d) : d(d) {} };
template <class D>
inline int ex_index2(const D &d, const vector<D> & v) {
	for(int i=0;i<v.size();i++) {
		if(d==v[i]) return i;
	}
	throw index2_failed_error<D>(d);
	return -1;
}



template <class D>
inline bool member(const D &d, const vector<D> & v) {
	return find(d,v)!=v.end();
}

template <class T>
void delete_and_erase(vector<T *> &v, int i) {
  delete v[i];
  v.erase(v.begin()+i);
}

template <class D>
struct not_found { D d; not_found(D d) : d(d) {} };

template <class K, class D>
D get_ex(const K &k, const map<K,D> &m) {
	map<K,D>::const_iterator i = m.find(k);
	if(i==m.end()) throw not_found<K>(k);
	return i->second;
}


}



namespace sven {

//shouldn't math lib have one of these?
template<class T>
inline int sign(const T &t) {
	return t<0 ? -1 : 1;
}

template<class T>
inline T abs(const T &t) {
	return t<0 ? -t : t;
}

//useful random number funcitons...
inline GLfloat randFloat(GLfloat range) {
	return (rand() / (GLfloat) RAND_MAX) * range;
}

//returns in [0,range)
inline GLint randInt(GLint range) {
	return rand()%(range+1);
}

inline float3 randVec(float3 range) {
	float3 rval(randFloat(range[0])-range[0]*.5, randFloat(range[1])-range[1]*.5, randFloat(range[2])-range[2]*.5);
	return rval;
}

inline bool chance(GLfloat p) {
	return rand() / (float) RAND_MAX <= p;
}

//simple color struct, works just like you'd expect
struct rgb {
	unsigned char r,g,b;
	rgb() {}
	rgb(unsigned char r, unsigned char g, unsigned char b) : r(r), g(g), b(b) {}
	rgb(unsigned char * p) : r(p[0]), g(p[1]), b(p[2]) {} 
	void set(unsigned char rs, unsigned char gs, unsigned char bs) {
		r=rs;g=gs;b=bs;
	}
};

//simple color struct, works just like you'd expect
struct rgba {
	unsigned char r,g,b,a;
	rgba() {r=0xff;g=0xff;b=0xff;a=0xff;}
	rgba(unsigned char r, unsigned char g, unsigned char b, unsigned char a) : r(r), g(g), b(b), a(a) {}
	rgba(unsigned char * p) : r(p[0]), g(p[1]), b(p[2]), a(p[3]) {} 
	void set(unsigned char rs, unsigned char gs, unsigned char bs, unsigned char as) {
		r=rs;g=gs;b=bs;a=as;
	}

	//easy interface with float4s
	rgba(float4 f) : r(f[0]), g(f[1]), b(f[2]), a(f[3]) {}
	operator float4() { return float4(r,g,b,a); }
};

//simple 2d point structs, work just like you'd expect
//I would almost just use 'typedef array[2] int2;' but
//then passing int2(1,1) wouldn't work....
struct int2 {
	int x,y;
	int2(int x, int y) : x(x), y(y) {}
	int &operator [](int i)			{ return i ? y : x; }
	int  operator [](int i) const	{ return i ? y : x; }
	int2() {}

	bool operator<(const int2 &b) const {
		return x<b.x || (x==b.x && y<b.x);
	}
};

struct int3 {
	int v[3];
	int3() {}
	int3(int v0, int v1, int v2) { v[0]=v0;v[1]=v1;v[2]=v2;}
	int3(int *vv) { v[0]=vv[0];v[1]=vv[1];v[2]=vv[2];}
	int &operator [](int i)		{ return v[i]; }
	int  operator [](int i) const	{ return v[i]; }
};

//stores top right, and bottom left corners of the rectangle
struct Int_Rect {
	int tx,ty,bx,by;	
	rgb color;


	friend std::ostream& operator<<(std::ostream &out, const Int_Rect &r) {
		using namespace std;
		out << "top: (" << r.tx << " , " << r.ty << 
		" ) bottom: ( " << r.bx << " , " << r.by <<  " )"; 
	return out;
}
};



//"Polygon" in a function in wingdi.h, so I need to name my polygons something else...
//just a list of verties.
typedef std::vector<float2> Float2_Polygon;
typedef std::vector<Float2_Polygon> Float2_PolygonList;


//////////////////////////////////////////
//Float_Rect 
//Specifies a rectangular area in 2 space
//by rembering the upper left hand corner (tx,ty)
//and lower right hand corner (bx,by) of the rectangle.
//Some functions that use Float_Rects assume tx<bx && ty>by.
struct Float_Rect {
	const static Float_Rect null; // (0,0,0,0)
	float tx,ty,bx,by;

	//force the tx<bx && ty>by assumption to hold.
	void force_standard_form() {
		using namespace std_ex;
		if(tx>bx) swap(tx,bx);
		if(ty<by) swap(ty,by);
	}

	///////////////////////////////////////////
	//make a rectangle from a polygon (returns null on failure), 
	//  result always in standard form
	friend Float_Rect convert(const Float2_Polygon & p);
	
	//standard constructor
	Float_Rect(float tx, float ty, float bx, float by) :
	tx(tx), ty(ty), bx(bx), by(by) {}

	//two handy functions
	float length() { return bx-tx; }
	float height() { return ty-by; }
	
	///////////////////////////////////////////
	//operators
	friend operator ==(const Float_Rect &a, const Float_Rect &b) {
		return a.tx==b.tx && a.ty==b.ty && a.bx==b.bx && a.by==b.by;
	}
	friend operator !=(const Float_Rect &a, const Float_Rect &b) {
		return !(a==b);
	}
	friend operator < (const Float_Rect &a, const Float_Rect &b) {
		return a.ty < b.ty;
	}


#pragma warning(disable: 4800) 
	operator bool() { return *this!=null; }

};


///////////////////////////////////
//Assosiated functions for rgb
inline bool operator==(const rgb &a, const rgb &b) {
	return a.r==b.r && a.g==b.g && a.b==b.b;
}

inline bool operator==(const rgba &a, const rgba &b) {
	return a.r==b.r && a.g==b.g && a.b==b.b && a.a==b.a;
}

inline bool operator!=(const rgb &a, const rgb &b) {
	return a.r!=b.r || a.g!=b.g || a.b!=b.b;
}

inline bool operator!=(const rgba &a, const rgba &b) {
	return a.r!=b.r || a.g!=b.g || a.b!=b.b || a.a!=b.a;
}

inline std::ostream& operator<<(std::ostream &out, const rgb &c) {
	out << "(" << (int)c.r << " " << (int)c.g << " " << (int)c.b << ")"; 
	return out;
}

inline std::ostream& operator<<(std::ostream &out, const rgba &c) {
	out << "(" << (int)c.r << " " << (int)c.g << " " << (int)c.b << " " << (int)c.a << ")"; 
	return out;
}

///////////////////////////////////
//Assosiated functions for int2
inline bool operator==(const int2 &a, const int2 &b) {
	return a.x==b.x && a.y==b.y;
}

inline bool operator!=(const int2 &a, const int2 &b) {
	return a.x!=b.x || a.y!=b.y;
}

inline std::ostream& operator<<(std::ostream &out, const int2 &p) {
	out << "(" << (int)p.x << " " << (int)p.y << ")"; 
	return out;
}

///////////////////////////////////
//Assosiated functions for float2
inline bool operator==(const float2 &a, const float2 &b) {
	return a.x==b.x && a.y==b.y;
}

inline bool operator!=(const float2 &a, const float2 &b) {
	return a.x!=b.x || a.y!=b.y;
}

inline std::ostream& operator<<(std::ostream &out, const float2 &p) {
	out << "(" << (int)p.x << " " << (int)p.y << ")"; 
	return out;
}


}



#endif