

//You can get a long way without turning on direct input 
//(by undefing defining USE_DINPUT)

//The g_mouse and the input_buffer[] are both maintained using the
//windows API (look at WI_Proc to see how that happens)

//The keys[] and mouseState are both maintained by direct input,
//so they will only be defined if USE_DINPUT is defined.
//Using them also requires that your main file first call initDirectInput.
//Call updateDirecInput every time you want the direct input data updated,
//call killDirectInput after you are done with dinput (typically at the end of the program).

//there are two character buffers: 
//one that is created by direct input, keys[], 
//it is very useful if you are interested in physical key input, such as quickly detecting
//arrow or function key presses.
//The other array holds a buffer that remembers all the text that has been typed.
//So if you want to do text input, clear the array, then wait for the user to fill it 
//with interesting characters.  The text input buffer is maintained using standard
//windows API stuff (it is updated in WI_Proc).


#ifndef WIN_INPUT_H
#define WIN_INPUT_H


#include <windows.h>		// Header File For Windows

//by default this is on.
#define USE_DINPUT 1
#define INPUT_BUFFER_LENGTH 300

#ifdef USE_DINPUT

#include <dinput.h>

namespace win_tools {

//by default joystick input is off.
//(At the moment my joystick code crashes if it
//can't detect a joystick.  As soon as I get around
//to fixing that bug, joystick support will be
//included by default along with dinput.  But
//untill then, this preprocessor trick will have to do.
//#define USE_JOYSTICK

//I considered having WinMouse inherit from DIMOUSESTATE,
//thus combining all mouse information into one easy to 
//use class.
//Unfortunatly, given the way that the direct input
//libraries work, that would force me to rely on
//object slicing to properly maintain the mouse state info.
//And object slicing is one of those things that I
//should probably avoid relying on....
extern DIMOUSESTATE  mouseState;

void updateDirectInput();
bool initDirectInput();
void killDirectInput();

struct DI_ResourceWrapper {
	DI_ResourceWrapper() { 	initDirectInput(); updateDirectInput(); }
	~DI_ResourceWrapper() { killDirectInput(); }
};

//direct state information for the keyboard and mouse
extern DIMOUSESTATE  mouseState;
extern BYTE keys[256];

#ifdef USE_JOYSTICK

void initJoyStick();
void killJoyStick();
DIJOYSTATE  getJSData();

#endif


}
#endif


namespace win_tools {

//lClick will return true the first time it is called after a ldMessage has
//been registered. (rClick works similarly)
//The x,y variables hold the mouse's x,y screen position.
//The check_lb_state() functions can be used to look at the left and right mouse button
//states without actually calling lClick.
//In my experience, you usually want to call lClick rather than check_lb_state.
class WinMouse 
{
private:
	bool lclick, rclick;
	short dz; //delta z, for the mouse wheel
	bool lb, rb;

public:
	bool check_lb_state() { return lb; }
	bool check_rb_state() { return rb; }
	
	int x, y;


	WinMouse() : x(0), y(0), lb(false), rb(false), lclick(false), rclick(false),
		dz(0) {}

	short getdz() { short rval=dz; dz=0; return rval; }
	void setdz(short z) { dz+=z; }

	void ldMessage() {
		lb=true;
		lclick=true;
	}
	void luMessage() {
		lb=false;
		lclick=false;
	}
	bool lClick() {
		if(lclick) { 
			lclick=false; 
			return true;
		}
		return false;
	}
	void rdMessage() {
		rb=true;
		rclick=true;
	}
	void ruMessage() {
		rb=false;
		rclick=false;
	}
	bool rClick() {
		if(rclick) { 
			rclick=false; 
			return true;
		}
		return false;
	}
};

extern WinMouse g_mouse;
extern char input_buffer[INPUT_BUFFER_LENGTH];

//to use the g_mouse, you need to use the WI_Proc as your winproc
//(alternatively, write your own winproc, breaking or ignoring input messages,
//then return WI_Proc).
LRESULT CALLBACK WI_Proc(HWND hWnd,UINT uMsg,WPARAM	wParam, LPARAM lParam);

}

#endif