#include "std_precomp.h"

#include <wx/colordlg.h>
#include <wx/choice.h>
#include <wx/spinctrl.h>
#include <wx/textctrl.h>
#include <wx/radiobox.h>
#include <wx/dir.h>
#include "ifl_wx.h"
#include <simple_regexp.h>

#include "stam.h"

const int SPARAM_DT		= 600;
const int SPARAM_DIFF	= 601;
const int SPARAM_VISC	= 602;
const int SPARAM_EPS	= 603;
const int SPARAM_DDECAY = 604;
const int SPARAM_DALPHA = 605;

const int NPARAM_MAX_AREA	= 610;
const int NPARAM_MIN_DIST	= 611;
const int NPARAM_BRUSH_WIDTH= 612;
const int NPARAM_MAX_BRL	= 613;
const int NPARAM_MIN_BRL	= 614;
const int NPARAM_BVR		= 615;

const int NPR_BCOLORS_RB    = 630;

//I actually considered using defines here to cut down the risk of variable name chomping...
const int NPARAM_MIN_INTENSITY_SCALE	= 640;
const int NPARAM_MAX_INTENSITY_SCALE	= 641;
const int NPARAM_RANDOM_WIDTH_MIN		= 642;
const int NPARAM_RANDOM_WIDTH_MAX		= 643;
const int NPARAM_DELTA_COLOR_MIN		= 644;
const int NPARAM_DELTA_COLOR_MAX		= 645;


//////////////////////////////////////////
//UI menu stuff
/////////////////////

namespace npr {
void clear_points();
}
struct UI_Stam : public UI_Abstract {

	void OnMouse(wxMouseEvent& event) {
		int x=event.GetX();
		int y=event.GetY();

		Stam::mouse_func(0,event.m_leftDown,x,y);
		Stam::mouse_func(1,event.m_middleDown,x,y);
		Stam::mouse_func(2,event.m_rightDown,x,y);
	}

	void OnChar(wxKeyEvent& event) {
		Stam::key_func(
			(unsigned char) event.m_keyCode,
			event.GetX(),
			event.GetY()
			);
		if((unsigned char) event.m_keyCode == 'q') {
			npr::clear_points();
		}
	}

};



UI_Bounds boundsUI;
UI_Stam stamUI;
UI_Abstract * currentUI=&stamUI;


wxMenu * UIMenu() {
  wxMenu *uiMenu = new wxMenu;
  
  uiMenu->Append(ID_STAM_UI, _T("&Stam Style\tCtrl-s"),"", TRUE);
  uiMenu->Append(ID_BOUNDS_UI, _T("&Insert Bounds\tCtrl-b"),"", TRUE);

  uiMenu->Check(ID_STAM_UI, true);
  return uiMenu;
}

void MyFrame::UI_Stam(wxCommandEvent& event) {
	GetMenuBar()->Check(ID_STAM_UI, true);
	GetMenuBar()->Check(ID_BOUNDS_UI, false);
	currentUI=&stamUI;	
}


void MyFrame::UI_Bounds(wxCommandEvent& event) {
	GetMenuBar()->Check(ID_STAM_UI, false);
	GetMenuBar()->Check(ID_BOUNDS_UI, true);
	currentUI=&boundsUI;
}

////////////////////////////////////////////////////
//Display menu stuff
////////////////////////////////////

wxMenu * ShowMenu() {
  wxMenu *showMenu = new wxMenu;
  
  showMenu->AppendCheckItem(SHOW_POINTS,    "&Show Points\tShift-p");
  showMenu->AppendCheckItem(SHOW_TRIANGLES, "&Show Triangles\tShift-t");
  showMenu->AppendCheckItem(SHOW_BRUSHES,   "&Show Brushes\tShift-r");
  showMenu->AppendCheckItem(SHOW_BOUNDS,    "&Show Bounds\tShift-b");
  showMenu->AppendCheckItem(SHOW_VELOCITY,  "&Show Velocity\tShift-v");
  showMenu->AppendCheckItem(SHOW_DENSITY,   "&Show Density\tShift-d");
  
using namespace display;

  showMenu->Check(SHOW_POINTS, show_points);
  showMenu->Check(SHOW_TRIANGLES, show_triangles);
  showMenu->Check(SHOW_BRUSHES, show_brushes);
  showMenu->Check(SHOW_BOUNDS, show_bounds);
  showMenu->Check(SHOW_VELOCITY, show_velocity);
  showMenu->Check(SHOW_DENSITY, show_density);

  return showMenu;
}

void MyFrame::ShowPoints(wxCommandEvent& event) {
	display::show_points=GetMenuBar()->IsChecked(event.GetId());
}
void MyFrame::ShowBrushes(wxCommandEvent& event) {
	display::show_brushes= GetMenuBar()->IsChecked(event.GetId());
}
void MyFrame::ShowTriangles(wxCommandEvent& event) {
	display::show_triangles=GetMenuBar()->IsChecked(event.GetId());
}
void MyFrame::ShowBounds(wxCommandEvent& event) {
	display::show_bounds=GetMenuBar()->IsChecked(event.GetId());
}
void MyFrame::ShowDensity(wxCommandEvent& event) {
	display::show_density=GetMenuBar()->IsChecked(event.GetId());
}
void MyFrame::ShowVelocity(wxCommandEvent& event) {
	display::show_velocity= GetMenuBar()->IsChecked(event.GetId());
}



///////////////////////////////////////////////////////
//NPR menu stuff
////////////////////////////////////////////////////

namespace npr {
	extern float max_area;
	extern float min_dist;
	extern float max_brush_len;
	extern float min_brush_len;
	extern float brush_vel_ratio;

	extern int delta_color_max;
	extern int delta_color_min;
	extern float scale_intensity_min;
	extern float scale_intensity_max;
	extern float random_width_min;
	extern float random_width_max;

}

wxMenu * NPRMenu() {
	wxMenu *nprMenu = new wxMenu;
	nprMenu->Append(NPR_INIT_POINTS, "&Init Points\tCtrl-i");
	nprMenu->Append(ID_NPR_PARAMS, "&Set NPR Params");
	nprMenu->Append(ID_NRAND_PARAMS, "&Random Perterbation Vars");
	nprMenu->Append(ID_NPR_BCOLORS, "&Brush Color Options");

	return nprMenu;
}

typedef void (BColor_fun)();

namespace npr {
	extern BColor_fun * brush_color;
	rgba white_brush(float x, float y);
	rgba red_brush(float x, float y);
	extern BColor_fun * bcolor_funcs[];

	wxString bcolor_names[]={"white","red", "density"};
}

struct NPR_BColorsDialog : public wxDialog {
	wxRadioBox * rb;

	NPR_BColorsDialog(wxWindow * parent) :
		wxDialog(parent,-1,"Brush Color Options", wxPoint(0,10), wxSize(160,160)) {
		rb= new wxRadioBox(this,NPR_BCOLORS_RB,"",wxPoint(-1,-1), wxSize(-1,-1),
			3,npr::bcolor_names,0,wxRA_SPECIFY_ROWS);
		Show();
	}

	void RBevent(wxCommandEvent &event) {
		npr::brush_color = npr::bcolor_funcs[rb->GetSelection()];
	}

DECLARE_EVENT_TABLE()
};

BEGIN_EVENT_TABLE(NPR_BColorsDialog, wxDialog)
	EVT_RADIOBOX(NPR_BCOLORS_RB,  NPR_BColorsDialog::RBevent)
END_EVENT_TABLE()

void MyFrame::NPR_BColors(wxCommandEvent& event)
{
   (void) new NPR_BColorsDialog(this);
}

//much like every other param-editting dialog I have ever written,
//only with longer variable names.
struct NPR_RandsDialog : public wxDialog {
	IntCtrl * delta_color_max;
	IntCtrl *  delta_color_min;
	FloatCtrl * scale_intensity_min;
	FloatCtrl * scale_intensity_max;
	FloatCtrl * random_width_min;
	FloatCtrl * random_width_max;

	NPR_RandsDialog(wxWindow * parent) :
	  wxDialog(parent,-1,"RandParam Editor", wxPoint(0,10), wxSize(160,160)) {

		(void) new wxStaticText(this, -1, "iscale min:", wxPoint(5,5));
		scale_intensity_min = new FloatCtrl(&npr::scale_intensity_min,this,NPARAM_MIN_INTENSITY_SCALE, wxPoint(65, 0), wxSize(80, -1));

		(void) new wxStaticText(this, -1, "iscale max:", wxPoint(5,25));
		scale_intensity_max = new FloatCtrl(&npr::scale_intensity_max,this,NPARAM_MAX_INTENSITY_SCALE,wxPoint(65, 20), wxSize(80, -1));

		(void) new wxStaticText(this, -1, "width min:", wxPoint(5,45));
		random_width_min = new FloatCtrl(&npr::random_width_min,this,NPARAM_RANDOM_WIDTH_MIN,wxPoint(65, 40), wxSize(80, -1));

		(void) new wxStaticText(this, -1, "width max:", wxPoint(5,65));
		random_width_max = new FloatCtrl(&npr::random_width_max,this,NPARAM_RANDOM_WIDTH_MAX,wxPoint(65, 60), wxSize(80, -1));

		(void) new wxStaticText(this, -1, "dcolor min:", wxPoint(5,85));
		delta_color_min = new IntCtrl(&npr::delta_color_min,this,NPARAM_DELTA_COLOR_MIN,wxPoint(65, 80), wxSize(80, -1));

		(void) new wxStaticText(this, -1, "dcolor max:", wxPoint(5,105));
		delta_color_max = new IntCtrl(&npr::delta_color_max,this,NPARAM_DELTA_COLOR_MAX,wxPoint(65, 100), wxSize(80, -1));
	  
	  	Show();
	  }

	  void Set_scale_intensity_min(wxCommandEvent &event) { scale_intensity_min->OnEnter(); }
	  void Set_scale_intensity_max(wxCommandEvent &event) { scale_intensity_max->OnEnter(); }

	  void Set_random_width_min(wxCommandEvent &event) { random_width_min->OnEnter(); }
	  void Set_random_width_max(wxCommandEvent &event) { random_width_max->OnEnter(); }

	  void Set_delta_color_min(wxCommandEvent &event) { delta_color_min->OnEnter(); }
	  void Set_delta_color_max(wxCommandEvent &event) { delta_color_max->OnEnter(); }


	  DECLARE_EVENT_TABLE()
};

//I need to change my conventions in these calls so that 
//I can't open up multiple copies of the same dialog
void MyFrame::NPR_RandParams(wxCommandEvent& event) {
  (void) new NPR_RandsDialog(this);
}

BEGIN_EVENT_TABLE(NPR_RandsDialog, wxDialog)
	EVT_TEXT_ENTER(NPARAM_MIN_INTENSITY_SCALE,  NPR_RandsDialog::Set_scale_intensity_min)
	EVT_TEXT_ENTER(NPARAM_MAX_INTENSITY_SCALE,  NPR_RandsDialog::Set_scale_intensity_max)

	EVT_TEXT_ENTER(NPARAM_RANDOM_WIDTH_MIN,  NPR_RandsDialog::Set_random_width_min)
	EVT_TEXT_ENTER(NPARAM_RANDOM_WIDTH_MAX,  NPR_RandsDialog::Set_random_width_max)

	EVT_TEXT_ENTER(NPARAM_DELTA_COLOR_MIN,  NPR_RandsDialog::Set_delta_color_min)
	EVT_TEXT_ENTER(NPARAM_DELTA_COLOR_MAX,  NPR_RandsDialog::Set_delta_color_max)

END_EVENT_TABLE()

//i really need to figure out a way to better automate the creation of
//these param editing dialogs...
//i am thinking it would just be something that took in a list of float*/label
//pairs
struct NPR_ParamsDialog : public wxDialog {

	FloatCtrl * min_dist;
	FloatCtrl * max_area;
//	FloatCtrl * brush_width;
	FloatCtrl * max_brush_len;
	FloatCtrl * min_brush_len;
	FloatCtrl * brush_vel_ratio;

	NPR_ParamsDialog(wxWindow * parent) :
	  wxDialog(parent,-1,"NPR Param Editor", wxPoint(0,10), wxSize(160,160)) {

		(void) new wxStaticText(this, -1, "min dist:", wxPoint(5,5));
		min_dist = new FloatCtrl(&npr::min_dist,this,NPARAM_MIN_DIST,wxPoint(65, 0), wxSize(80, -1));

		(void) new wxStaticText(this, -1, "max area:", wxPoint(5,25));
		max_area = new FloatCtrl(&npr::max_area,this,NPARAM_MAX_AREA,wxPoint(65, 20), wxSize(80, -1));

		(void) new wxStaticText(this, -1, "max brlen:", wxPoint(5,45));
		max_brush_len = new FloatCtrl(&npr::max_brush_len,this,NPARAM_MAX_BRL,wxPoint(65, 40), wxSize(80, -1));

		(void) new wxStaticText(this, -1, "min brlen:", wxPoint(5,65));
		min_brush_len = new FloatCtrl(&npr::min_brush_len,this,NPARAM_MIN_BRL,wxPoint(65, 60), wxSize(80, -1));

		(void) new wxStaticText(this, -1, "br_len/vel:", wxPoint(5,85));
		brush_vel_ratio = new FloatCtrl(&npr::brush_vel_ratio,this,NPARAM_BVR,wxPoint(65, 80), wxSize(80, -1));
		
		Show();	  
	  }

	  void Set_min_dist(wxCommandEvent &event) { min_dist->OnEnter(); }
	  void Set_max_area(wxCommandEvent &event) { max_area->OnEnter(); }
//	  void Set_brush_width(wxCommandEvent &event) { brush_width->OnEnter(); }
	  void Set_max_brush_len(wxCommandEvent &event) { max_brush_len->OnEnter(); }
	  void Set_min_brush_len(wxCommandEvent &event) { min_brush_len->OnEnter(); }
	  void Set_brush_vel_ratio(wxCommandEvent &event) { brush_vel_ratio->OnEnter(); }


DECLARE_EVENT_TABLE()
};

BEGIN_EVENT_TABLE(NPR_ParamsDialog, wxDialog)
	EVT_TEXT_ENTER(NPARAM_MIN_DIST,  NPR_ParamsDialog::Set_min_dist)
	EVT_TEXT_ENTER(NPARAM_MAX_AREA,  NPR_ParamsDialog::Set_max_area)
//	EVT_TEXT_ENTER(NPARAM_BRUSH_WIDTH,  NPR_ParamsDialog::Set_brush_width)
	EVT_TEXT_ENTER(NPARAM_MAX_BRL,  NPR_ParamsDialog::Set_max_brush_len)
	EVT_TEXT_ENTER(NPARAM_MIN_BRL,  NPR_ParamsDialog::Set_min_brush_len)
	EVT_TEXT_ENTER(NPARAM_BVR,  NPR_ParamsDialog::Set_brush_vel_ratio)
END_EVENT_TABLE()


void MyFrame::NPR_Params(wxCommandEvent& event)
{
  (void) new NPR_ParamsDialog(this);
}


///////////////////////////////////////////////////////
//Fluid Menu stuff
///////////////////////////////////////////////////////

wxMenu * FluidMenu() {
  wxMenu *fluidMenu = new wxMenu;
  
  fluidMenu->Append(ID_SOLVER_PARAMS, "&Solver Paramaters\tCtrl-d");
  fluidMenu->Append(ID_HAULT_SIM, "&Hault/Resume Sim\tCtrl-h");
  fluidMenu->Append(ID_SET_BOUND_COLOR, "&Set Bound Color");
  return fluidMenu;
}

void MyFrame::BoundColor(wxCommandEvent& event) {
	using Stam::bound_color;
	wxColour bc(bound_color.r,bound_color.g,bound_color.b);
	wxColourData data;
	data.SetColour(bc);
	wxColourDialog dialog(this, &data);
 
    dialog.SetTitle("Choose the bounding block color");
    if (dialog.ShowModal() == wxID_OK)
    {
        wxColour c = dialog.GetColourData().GetColour();
        bound_color.r=c.Red();
        bound_color.g=c.Green();
        bound_color.b=c.Blue();
    }

}

struct SolverParamsDialog : public wxDialog {

	FloatCtrl * dt;
	FloatCtrl * diff;
	FloatCtrl * visc;
	FloatCtrl * epsilon;
	FloatCtrl * dalpha;
	FloatCtrl * ddecay;

	SolverParamsDialog(wxWindow * parent) :
	  wxDialog(parent,-1,"Param Editor", wxPoint(0,10), wxSize(150,150)) {

		(void) new wxStaticText(this, -1, "dt:", wxPoint(5,5));
		dt = new FloatCtrl(&Stam::dt,this,SPARAM_DT,wxPoint(50, 0), wxSize(80, -1));

		(void) new wxStaticText(this, -1, "diff:", wxPoint(5,25));
		diff = new FloatCtrl(&Stam::diff,this,SPARAM_DIFF,wxPoint(50, 20), wxSize(80, -1));

		(void) new wxStaticText(this, -1, "visc:", wxPoint(5,45));
		visc = new FloatCtrl(&Stam::visc,this,SPARAM_VISC,wxPoint(50, 40), wxSize(80, -1));

		(void) new wxStaticText(this, -1, "epsilon", wxPoint(5,65));
		epsilon = new FloatCtrl(&Stam::epsilon,this,SPARAM_EPS,wxPoint(50, 60), wxSize(80, -1));

//
		(void) new wxStaticText(this, -1, "d-alpha", wxPoint(5,85));
		dalpha = new FloatCtrl(&Stam::dalpha,this,SPARAM_DALPHA,wxPoint(50, 80), wxSize(80, -1));


		(void) new wxStaticText(this, -1, "d-decay", wxPoint(5,105));
		ddecay = new FloatCtrl(&Stam::ddecay,this,SPARAM_DDECAY,wxPoint(50, 100), wxSize(80, -1));

		Show();	  
	  }





	  void Set_dt(wxCommandEvent &event) { dt->OnEnter(); }
	  void Set_diff(wxCommandEvent &event) { diff->OnEnter(); }
	  void Set_visc(wxCommandEvent &event) { visc->OnEnter(); }
	  void Set_epsilon(wxCommandEvent &event) { epsilon->OnEnter(); }
	  void Set_ddecay(wxCommandEvent &event) { ddecay->OnEnter(); }
	  void Set_dalpha(wxCommandEvent &event) { dalpha->OnEnter(); }

DECLARE_EVENT_TABLE()
};

void MyFrame::SolverParams(wxCommandEvent& event)
{
    (void) new SolverParamsDialog(this);
}

BEGIN_EVENT_TABLE(SolverParamsDialog, wxDialog)
	EVT_TEXT_ENTER(SPARAM_DT,    SolverParamsDialog::Set_dt)
	EVT_TEXT_ENTER(SPARAM_DIFF,  SolverParamsDialog::Set_diff)
	EVT_TEXT_ENTER(SPARAM_VISC,  SolverParamsDialog::Set_visc)
	EVT_TEXT_ENTER(SPARAM_EPS,   SolverParamsDialog::Set_epsilon)
	EVT_TEXT_ENTER(SPARAM_DDECAY,   SolverParamsDialog::Set_ddecay)
	EVT_TEXT_ENTER(SPARAM_DALPHA,   SolverParamsDialog::Set_dalpha)
END_EVENT_TABLE()

