//This program was created using Julian Smart's GL Cube program as a base.

#ifndef _WX_CUBE_H_
#define _WX_CUBE_H_

#include "wx/glcanvas.h"
#include <particle.h>

// Define a new application type
class MyApp: public wxApp
{
public:
    bool OnInit(void);
};

// Define a new frame type
class TestGLCanvas;
class MyFrame: public wxFrame
{
public:


    MyFrame(wxFrame *frame, const wxString& title, const wxPoint& pos,
		const wxSize& size, long style = wxDEFAULT_FRAME_STYLE);

    void OnExit(wxCommandEvent& event);
    void OnPrint(wxCommandEvent& event);
    void OnBatchPrint(wxCommandEvent& event);
    void OnResize(wxCommandEvent& event);
	void OnSize(wxSizeEvent &event);

	//fluid menu
	void SolverParams(wxCommandEvent& event);
	void BoundColor(wxCommandEvent& event);
	void HaultSim(wxCommandEvent& event);

	//UI modes
	void UI_Stam(wxCommandEvent& event);
	void UI_Bounds(wxCommandEvent& event);

	//NPR menu
	void NPR_InitPoints(wxCommandEvent& event);
	void NPR_Params(wxCommandEvent& event);
	void NPR_BColors(wxCommandEvent& event);
	void NPR_RandParams(wxCommandEvent& event);

	//NPR vis options
	void ShowPoints(wxCommandEvent& event);
	void ShowBrushes(wxCommandEvent& event);
	void ShowTriangles(wxCommandEvent& event);

	//Standard fluid vis options
	void ShowDensity(wxCommandEvent& event);
	void ShowVelocity(wxCommandEvent& event);
	void ShowBounds(wxCommandEvent& event);


public:
    TestGLCanvas*    m_canvas;

DECLARE_EVENT_TABLE()
};

class TestGLCanvas: public wxGLCanvas
{
  friend class MyFrame;
public:

	bool batch_printing;
	long batch_prints;
	long batch_print_size;
	std::string batch_print_fname;
	effects::StandardEmitter<> * p_gen;

 TestGLCanvas(wxWindow *parent, const wxWindowID id = -1, 
    const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize,
    long style = 0, const wxString& name = "TestGLCanvas", int *attribList=0);
   
 ~TestGLCanvas(void);

 void OnPaint(wxPaintEvent& event);
 void OnSize(wxSizeEvent& event);
 void OnEraseBackground(wxEraseEvent& event);
 void OnEnterWindow( wxMouseEvent& event );
 void OnIdle( wxIdleEvent& event );
 void OnMouse( wxMouseEvent& event );
 void OnChar(wxKeyEvent& event);	

 void Render( void );
 void InitGL(void);
 void gl_draw();

 void setPersp(const int p);
 void gl_print_screen(const char * fname);

private:
  bool   m_init;
  GLuint m_gllist;
  long   m_rleft;
  long   m_rright;

  static unsigned long  m_secbase;
  static int            m_TimeInitialized;
  static unsigned long  m_xsynct;
  static unsigned long  m_gsynct;
 
  long           m_Key;
  unsigned long  m_StartTime;
  unsigned long  m_LastTime;
  unsigned long  m_LastRedraw;

DECLARE_EVENT_TABLE()
};

//this really ought to get put in a standard wx_ex library or something..
struct FloatCtrl : public wxTextCtrl {
	float * fp;
	FloatCtrl(float * fp, wxWindow* parent, wxWindowID id, const wxPoint& pos, 
		const wxSize& size) : fp(fp), wxTextCtrl(parent,id, "", pos, size, wxTE_PROCESS_ENTER ) {
		
		//It seems that if i don't do this, the wxString will chomp
		//my float to 2 decimal places...
		ostringstream f;
		f<<*fp;

		(*this) << f.str().c_str();
	}

	void OnEnter() {
		using namespace boost_ex;
		string v(GetValue().c_str());
		if( is_float(v) ) {
			*fp=make_float(v);
		}
		else {
			SetValue("");
			(*this) << *fp;
		}
	}
};

struct IntCtrl : public wxTextCtrl {
	int * ip;
	IntCtrl(int * ip, wxWindow* parent, wxWindowID id, const wxPoint& pos, 
		const wxSize& size) : ip(ip), wxTextCtrl(parent,id, "", pos, size, wxTE_PROCESS_ENTER ) {
		
		//It seems that if i don't do this, the wxString will chomp
		//my float to 2 decimal places...
		ostringstream i;
		i<<*ip;

		(*this) << i.str().c_str();
	}

	void OnEnter() {
		using namespace boost_ex;
		string v(GetValue().c_str());
		if( is_int(v) ) {
			*ip=make_float(v);
		}
		else {
			SetValue("");
			(*this) << *ip;
		}
	}
};


extern bool hault;

const int ID_RESIZE = 130;

//main menu stuff

//const int ID_FILE_LOAD = 131;
const int ID_PRINT_SCREEN = 132;
const int ID_BATCH_PRINT = 133;



const int SHOW_POINTS=160;
const int SHOW_TRIANGLES=161;
const int SHOW_BRUSHES=162;
const int SHOW_VELOCITY=163;
const int SHOW_DENSITY=164;
const int SHOW_BOUNDS=165;

//npr menu stuff
const int NPR_INIT_POINTS=170;
const int ID_NPR_PARAMS  =171;
const int ID_NPR_BCOLORS =172;
const int ID_NRAND_PARAMS=183;

//fluid sim stuff
const int ID_SOLVER_PARAMS=190;
const int ID_HAULT_SIM= 191;
const int ID_SET_BOUND_COLOR = 192;

//UI options
const int ID_STAM_UI=210;
const int ID_BOUNDS_UI=211;


namespace npr {
	void display();
	void idle();
}

#endif

